M_VALS = "4240208, 2958361, 4302039, 6585157, 12233562, 12528820, 12692347, 12170175, 9010629, 9130385, 7738586, 5570117"
M_DAYS = "31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31"
W_HEIGHT = 400

RUBYSCRIPT2EXE_DLLS = ["SDL.dll", "SDL_gfx.dll", "msvcr80.dll"]

# #########################################################
# Implementation

class WaterValues < Array
	def initialize(months, days)
		i = -1
		@month_vals = months.map { |v| i+=1; v/(1000*24*days[i]) }
		@factor = (W_HEIGHT / @month_vals.max) *0.9
		@days = days
		self << @month_vals.zip(days).inject([]) { |v, arr| v << [arr[0]]*arr[1] }
		flatten!
	end
	def [](i)
		return super unless i.kind_of? Numeric
		return if length < 1
		i+= length while i<0
		i-= length while i>=length
		super i
	end
	def avg(i)
		((self[i-2]+self[i+2])*1 + (self[i-1]+self[i+1])*5 + self[i]*18) / 30
	end
	def smooth()
		each_index do |i|
			self[i] = avg(i)
		end
	end
	def mstart(month)
		r = 0
		(month).times { |i| r+= @days[i] }; r
	end
	def each_month
		12.times { |m|	yield self[mstart(m)...(mstart(m)+@days[m])], m }
	end
	def correct(m, value)
		mstart(m).upto(mstart(m)+@days[m]) { |i| self[i]-=value }
	end
	def correct_area()
		each_month do |mdays, i|
			average = mdays.inject { |s, n| s+n } / mdays.length.to_f
			correction = (average - @month_vals[i]) / mdays.length.to_f
			correct(i, correction*5)
		end
	end
	def draw(target, color)
		each_with_index { |v,x| target.vertical_line( [x,W_HEIGHT-v*@factor], W_HEIGHT, color) }
	end
end

def reset_to(vals, day_vals)
	regex = /([0-9]+\.?,?[0-9]*)[^0-9]*/
	months = vals.scan(regex).flatten.map { |v| v.to_f }
	days = day_vals.scan(regex).flatten.map { |v| v.to_i }
	$dayvals = WaterValues.new months, days
	$val_archive = [$dayvals.dup]
	$current_it = 0
	$max_it = 0
end
reset_to M_VALS, M_DAYS
def back(n=1)
	$current_it -= n
	$current_it = 0 if $current_it < 0
	visu
end
def forward(n=1)	
	$current_it += n
	if $current_it > $max_it
		tmp = $current_it - $max_it
		$current_it = $max_it
		iterate tmp
	end
	visu
end

def iterate(num)
	num.times do
		# Smooth
		3.times do
			$dayvals.smooth
		end
		# Correct Area
		$dayvals.correct_area
		# Save Result, increment counter
		$val_archive << $dayvals.dup
		$current_it += 1
		$max_it +=1
		# Visualize
		visu
	end
end

# #########################################################
# Gui Stuff

# Visualization Window
require 'RUDL'
include RUDL
$ds = DisplaySurface.new([365, W_HEIGHT])
$ds.set_caption("Wasser Visualisierung")

def visu()
	$ds.fill([0,0,0])
	$val_archive[$current_it].draw($ds, [100,100,255])
	$val_archive[0].draw($ds, [220,220,220,120])
	$ds.flip
	Timer.delay 10
	$status.text = "Iteration #$current_it von #$max_it"
end

# Entry Window
require 'tk'
root = TkRoot.new() { title "Wasser (c) 2005 Peter Thoman" }
TkClipboard.clear

$status = TkLabel.new(root).pack("side"=>"bottom")

# Buttons at the bottom
b_frame = TkFrame.new(root).pack(:side=>:bottom, :fill=>:y)
buttons = [["<<", proc { back(5) }], ["<", proc { back }], 
	["10 Iterationen", proc { $current_it = $max_it; iterate 10 }], 
	[">", proc { forward }], [">>", proc { forward(5) }]].map { |v|
		TkButton.new(b_frame) { 
			text v[0]
			command v[1]
		}
	}
TkGrid.configure(*buttons)

# Entry stuff
entry_frame = TkFrame.new(root).pack(:side=>:top)
TkLabel.new(entry_frame) { text "Monatswerte:" }.pack
text_months = TkText.new(entry_frame, :width=>60, :height=>4).pack
text_months.insert(:end, M_VALS)
TkLabel.new(entry_frame) { text "Monatstage:" }.pack
text_mdays = TkText.new(entry_frame, :width=>60, :height=>2).pack
text_mdays.insert(:end, M_DAYS)
entry_b_frame = TkFrame.new(entry_frame).pack
b_entry_load = TkButton.new(entry_b_frame) { 
		text "Werte laden (Reset)" 
		command proc {
			text_months.value = text_months.value.gsub(/\./,'').gsub(/\n[ ]*/,", ").chomp(", ")
			reset_to(text_months.value, text_mdays.value)
			visu
		}
	}
b_entry_clear = TkButton.new(entry_b_frame) { 
		text "Monatswerte lschen" 
		command proc {
			text_months.value = ""
		}
	}
TkGrid.configure(b_entry_clear, b_entry_load)
TkLabel.new(entry_frame) { text "---" }.pack

# Output
TkLabel.new(entry_frame) { text "Resultat:" }.pack
out_frame = TkFrame.new(entry_frame).pack
text_result = TkText.new(out_frame, :width=>60, :height=>12).pack('side'=>'left')
scroll_result = TkScrollbar.new(out_frame).pack('side'=>'right', 'fill'=>'y')
text_result.yscrollbar(scroll_result)
out_b_frame = TkFrame.new(entry_frame).pack
b_res_out = TkButton.new(out_b_frame) { 
		text "Resultat ausgeben"
		command proc {
			text_result.value = ""
			$val_archive[$current_it].each { |v|
				text_result.value += v.to_s.sub(/\./,",") + "\n"
			}
		}
	}
b_res_copy = TkButton.new(out_b_frame) { 
		text "Resultat kopieren"
		command proc {
			b_res_out.invoke
			TkClipboard.set(text_result.value)
		}
	}
TkGrid.configure(b_res_out, b_res_copy)
TkLabel.new(entry_frame) { text "---" }.pack

visu
Tk.mainloop()